{-
DPH dodaje novi tip podataka u Haskell - paralelne nizove, kao i mnogobrojne operacije sa njima.
Oznake: 
[: e :] -> paralelni niz elemenata tipa e
[: x, y, z :] -> paralelni niz sa tri elementa x, y i z
[: x + 1 | x <- xs :] -> generator za paralelni niz

Funkcije: P varijante vec pomenutih funkcija iz standardnih biblioteka, lengthP, sumP, mapP, filterP ... 
-}

-- sintaksa za paralelne nizove se omogucava jezicnom opcijom ParallelArrays
{-# LANGUAGE ParallelArrays #-}

-- potrebno je eksplicitno naglasiti kompajleru da zelimo vektorizovan modul (da bi uopste mogli da koristimo paralelne nizove za paralelizaciju nasih podataka )
{-# OPTIONS_GHC -fvectorise #-}
 
module SK (suma_omot)
where

{-
DPH ne podrzava klase vec za svaki tip ima definisan zaseban modul koji ukljucujemo u zavisnosti od funkcija koje su nam potrebne.
 
DPH moduli
- Data.Array.Parallel: http://hackage.haskell.org/package/dph-par-0.5.1.1/docs/Data-Array-Parallel.html

- Data.Array.Parallel.Prelude: http://hackage.haskell.org/package/dph-par-0.5.1.1/docs/Data-Array-Parallel-Prelude.html

- Data.Array.Parallel.Prelude.Int: http://hackage.haskell.org/package/dph-par-0.5.1.1/docs/Data-Array-Parallel-Prelude-Int.html

- Data.Array.Parallel.Prelude.Double: http://hackage.haskell.org/package/dph-par-0.5.1.1/docs/Data-Array-Parallel-Prelude-Double.html

- Data.Array.Parallel.PArray: http://hackage.haskell.org/package/dph-par-0.5.1.1/docs/Data-Array-Parallel-PArray.html
-}

import Data.Array.Parallel
import Data.Array.Parallel.Prelude
import Data.Array.Parallel.Prelude.Double
 
-- funkcija racuna sumu kvadrata elemenata vektora
-- obratiti paznju da koristimo operaciju mnozenja definisanu bas u modulu za tip Double 
suma_kv :: [:Double:] -> Double
suma_kv xs = sumP [:x Data.Array.Parallel.Prelude.Double.* x | x <- xs :]
 
{-
Bitan deo je interfejs izmedju vektorizovanog koda (koda u kom koristimo paralelne nizove) i nevektorizovanog koda (main funkcija koja ucitava podatke ciju obradu treba paralelizovati, klasican haskell kod). DPH je obezbedio prelazni tip PArray i funkciju za konverziju fromPArrayP koja je specifikovana da nevektorizovan oblik prevede u vektorizovan. Bitno je imati u vidu da ovaj tip nije vektorizovan, kao ni funkcije koje rade sa njim sto nam daje mogucnost da ga koristimo iz nevektorizovanog koda, odnosno main funkcije 
-}
-- da se ne bi povezivala sa nevektorizovanim kodom 
{-# NOINLINE suma_omot #-}
suma_omot :: PArray Double -> Double
-- prvo vrsimo konverziju podataka u vektorizovan niz, a zatim pozivamo glavnu funkciju 
suma_omot v  = suma_kv (fromPArrayP v)

